@@ -1,16 +1,15 @@
-# $Id: __init__.py 63 2010-03-26 02:20:33Z dugsong $
-
 """fast, simple packet creation and parsing."""
 
-__author__ = 'Dug Song <dugsong@monkey.org>'
-__copyright__ = 'Copyright (c) 2004 Dug Song'
+__author__ = 'Dug Song'
+__author_email__ = 'dugsong@monkey.org'
 __license__ = 'BSD'
 __url__ = 'http://dpkt.googlecode.com/'
-__version__ = '1.7'
+__version__ = '1.8.8'
 
 from dpkt import *
 
 import ah
+import aoe
 import aim
 import arp
 import asn1
@@ -34,6 +33,7 @@ import igmp
 import ip
 import ip6
 import ipx
+import llc
 import loopback
 import mrt
 import netbios
@@ -41,6 +41,7 @@ import netflow
 import ntp
 import ospf
 import pcap
+import pcapng
 import pim
 import pmap
 import ppp
