@@ -1,19 +1,22 @@
-# $Id: loopback.py 64 2005-01-26 19:13:17Z dugsong $
-
+# $Id: sll.py 23 2006-11-08 15:45:33Z dugsong $
+# -*- coding: utf-8 -*-
 """Linux libpcap "cooked" capture encapsulation."""
 
-import arp, dpkt, ethernet
+import arp
+import dpkt
+import ethernet
+
 
 class SLL(dpkt.Packet):
     __hdr__ = (
-        ('type', 'H', 0), # 0: to us, 1: bcast, 2: mcast, 3: other, 4: from us
+        ('type', 'H', 0),  # 0: to us, 1: bcast, 2: mcast, 3: other, 4: from us
         ('hrd', 'H', arp.ARP_HRD_ETH),
-        ('hlen', 'H', 6),	# hardware address length
-        ('hdr', '8s', ''),	# first 8 bytes of link-layer header
+        ('hlen', 'H', 6),  # hardware address length
+        ('hdr', '8s', ''),  # first 8 bytes of link-layer header
         ('ethtype', 'H', ethernet.ETH_TYPE_IP),
-        )
+    )
     _typesw = ethernet.Ethernet._typesw
-    
+
     def unpack(self, buf):
         dpkt.Packet.unpack(self, buf)
         try:
