@@ -1,10 +1,12 @@
-# $Id: udp.py 290 2006-01-22 02:43:28Z dugsong $
-
+# $Id: udp.py 23 2006-11-08 15:45:33Z dugsong $
+# -*- coding: utf-8 -*-
 """User Datagram Protocol."""
 
 import dpkt
 
-UDP_PORT_MAX	= 65535
+UDP_HDR_LEN = 8
+UDP_PORT_MAX = 65535
+
 
 class UDP(dpkt.Packet):
     __hdr__ = (
@@ -12,4 +14,4 @@ class UDP(dpkt.Packet):
         ('dport', 'H', 0),
         ('ulen', 'H', 8),
         ('sum', 'H', 0)
-        )
+    )
